searchData={"items":[{"type":"module","title":"BitwiseIp","doc":"A struct representing an IP address encoded as an integer.\n\nThe [Internet Protocol](https://en.wikipedia.org/wiki/Internet_Protocol)\ndefines computer network addresses using fixed-width integers, which are\nefficient for both transmission and the implementation of logic using bitwise\noperations. [IPv4](https://en.wikipedia.org/wiki/IPv4) uses 32-bit integers,\nproviding a space of 4,294,967,296 unique addresses. Due to the growing size\nof the internet, [IPv6](https://en.wikipedia.org/wiki/IPv6) uses 128-bit\nintegers to provide an absurdly large address space.\n\nThese integers, however, are hard for humans to read. Therefore, we've\nadopted customary notations that are a little easier to digest. IPv4 uses a\ndotted octet notation, where each of the four bytes are written in decimal\nnotation and separated by `.`, as in `127.0.0.1`. IPv6 is similar, but uses\nhexadecimal notation on each of eight hextets separated by `:`, as in\n`a:1:b:2:c:3:d:4`.\n\nAs such, representations for IP addresses in modern software have drifted\naway from fixed-width integers. `:inet` represents IP addresses as tuples\nlike `{127, 0, 0, 1}` for IPv4 and `{0xA, 1, 0xB, 2, 0xC, 3, 0xD, 4}` for\nIPv6. These are less efficient in both the space to store the addresses and\nthe time it takes to perform various operations. For example, whereas\ncomparing two 32-bit IPv4 addresses is typically one machine instruction,\ncomparing two tuples involves memory indirection for the tuple layout and 4\nseparate integer comparisons. This could be even worse if you represent IPs\nas strings in their human-readable format.\n\nThe difference is probably negligible for your application. In fact, Elixir &\nErlang don't have great support for fixed-width integer representations (see\n`t:t/0` for details). But in the interest of getting back to basics,\n`BitwiseIp` provides the missing interface for manipulating IP addresses as\nthe integers they were designed to be. This makes certain logic much easier\nto express and improves micro-benchmarks compared to tuple-based libraries\n(for whatever that's worth). The most useful functionality is in\n`BitwiseIp.Block`, which represents a\n[CIDR](https://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing) block.\nHowever, `BitwiseIp` is the fundamental structure that `BitwiseIp.Block` is\nbuilt on.","ref":"BitwiseIp.html"},{"type":"function","title":"BitwiseIp.decode/1","doc":"Decodes a bitwise IP into an `:inet`-style tuple.\n\nThe Erlang standard library represents IP addresses as tuples of integers: 4\noctet values for IPv4, 8 hextet values for IPv6. This function decodes the\nsingle number from a `BitwiseIp` struct into its constituent parts. This can\nbe undone with `encode/1`.\n\nBeware of redundant usage in performance-critical paths. Because of the\noverhead in decoding the integer, excessive translation back & forth between\nthe formats may outweigh any benefits gained from other operations on the\nsingle-integer representation.","ref":"BitwiseIp.html#decode/1"},{"type":"function","title":"Examples - BitwiseIp.decode/1","doc":"```\niex> BitwiseIp.decode(%BitwiseIp{proto: :v4, addr: 2130706433})\n{127, 0, 0, 1}\n\niex> BitwiseIp.decode(%BitwiseIp{proto: :v6, addr: 1})\n{0, 0, 0, 0, 0, 0, 0, 1}\n```","ref":"BitwiseIp.html#decode/1-examples"},{"type":"function","title":"BitwiseIp.encode/1","doc":"Encodes an `:inet`-style tuple as a bitwise IP.\n\nThe Erlang standard library represents IP addresses as tuples of integers: 4\noctet values for IPv4, 8 hextet values for IPv6. This function encodes the\nseparate values as a single number, which gets wrapped into a `BitwiseIp`\nstruct. This can be undone with `decode/1`.\n\nBeware of redundant usage in performance-critical paths. Because of the\noverhead in encoding the integer, excessive translation back & forth between\nthe formats may outweigh any benefits gained from other operations on the\nsingle-integer representation.","ref":"BitwiseIp.html#encode/1"},{"type":"function","title":"Examples - BitwiseIp.encode/1","doc":"```\niex> BitwiseIp.encode({127, 0, 0, 1})\n%BitwiseIp{proto: :v4, addr: 2130706433}\n\niex> BitwiseIp.encode({0, 0, 0, 0, 0, 0, 0, 1})\n%BitwiseIp{proto: :v6, addr: 1}\n```","ref":"BitwiseIp.html#encode/1-examples"},{"type":"function","title":"BitwiseIp.parse/1","doc":"Parses a string into a bitwise IP.\n\nThis function parses IPv4 and IPv6 strings in their respective notations and\nproduces an encoded `BitwiseIp` struct. This is done in an error-safe way by\nreturning a tagged tuple. To raise an error, use `parse!/1` instead.\n\n`BitwiseIp` implements the `String.Chars` protocol, so parsing can be undone\nusing `to_string/1`.","ref":"BitwiseIp.html#parse/1"},{"type":"function","title":"Examples - BitwiseIp.parse/1","doc":"```\niex> BitwiseIp.parse(\"127.0.0.1\")\n{:ok, %BitwiseIp{proto: :v4, addr: 2130706433}}\n\niex> BitwiseIp.parse(\"::1\")\n{:ok, %BitwiseIp{proto: :v6, addr: 1}}\n\niex> BitwiseIp.parse(\"not an ip\")\n{:error, \"Invalid IP address \\\"not an ip\\\"\"}\n\niex> BitwiseIp.parse(\"192.168.0.1\") |> elem(1) |> to_string()\n\"192.168.0.1\"\n\niex> BitwiseIp.parse(\"fc00::\") |> elem(1) |> to_string()\n\"fc00::\"\n```","ref":"BitwiseIp.html#parse/1-examples"},{"type":"function","title":"BitwiseIp.parse!/1","doc":"An error-raising variant of `parse/1`.\n\nThis function parses IPv4 and IPv6 strings in their respective notations and\nproduces an encoded `BitwiseIp` struct. If the string is invalid, it raises\nan `ArgumentError`.\n\n`BitwiseIp` implements the `String.Chars` protocol, so parsing can be undone\nusing `to_string/1`.","ref":"BitwiseIp.html#parse!/1"},{"type":"function","title":"Examples - BitwiseIp.parse!/1","doc":"```\niex> BitwiseIp.parse!(\"127.0.0.1\")\n%BitwiseIp{proto: :v4, addr: 2130706433}\n\niex> BitwiseIp.parse!(\"::1\")\n%BitwiseIp{proto: :v6, addr: 1}\n\niex> BitwiseIp.parse!(\"not an ip\")\n** (ArgumentError) Invalid IP address \"not an ip\"\n\niex> BitwiseIp.parse!(\"192.168.0.1\") |> to_string()\n\"192.168.0.1\"\n\niex> BitwiseIp.parse!(\"fc00::\") |> to_string()\n\"fc00::\"\n```","ref":"BitwiseIp.html#parse!/1-examples"},{"type":"type","title":"BitwiseIp.t/0","doc":"An integer-encoded IP address.\n\nThis type takes on two different shapes depending on the IP protocol. The\nsupported protocols are IPv4 and IPv6.\n\nNormally, the distinction would be down to the number of bits in a\nfixed-width integer representation. However, the Erlang VM doesn't support\nfixed-width integers, so there's no way to tell IPv4 addresses apart from\nIPv6 addresses using just a number. Therefore, this type is a struct with two\nfields:\n\n* `:proto` - the protocol, either `:v4` or `:v6`\n* `:addr` - the integer encoding of the address\n\nBut again, the VM does not support fixed-width integers for the `:addr`. In\nthe Erlang runtime system, the smallest unit of memory is a *word*: 4 bytes\non a 32-bit architecture, 8 bytes on a 64-bit architecture. Data is stored\nusing *tagged pointers*, where one word has 4 bits reserved as a *tag*\nenumerating type information. One pattern of 4 bits says \"I'm a float\",\nanother pattern says \"I'm an integer\", and so on. When the data is small\nenough to fit in the remaining bits of the word (28 bits or 60 bits,\ndepending on the architecture), it is stored as an *immediate* value.\nOtherwise, it is *boxed* and the word instead contains a pointer to a section\nof memory on the heap, which can basically be arbitrarily large. Read more in\n[*A staged tag scheme for\nErlang*](http://www.it.uu.se/research/publications/reports/2000-029/) by\nMikael Pettersson.\n\nWhat this means for us is that `:addr` may or may not spill onto the heap. On\na 32-bit machine, only IP addresses in the range of 0 to 2^28 fit as\nimmediate values. This covers most of the IPv4 range, but only a small\nportion of the IPv6 range. 64-bit machines have 60 bits to play with, which\nwould comfortably fit any IPv4 address, but still requires boxing of IPv6\naddresses. According to the [Erlang efficiency\nguide](http://erlang.org/doc/efficiency_guide/advanced.html), large integers\nare stored across at least 3 words. What's more, because we have to\ndistinguish between integers using the struct with the `:proto` field, each\nIP address requires an additional map allocation, which carries some\noverhead.\n\nSo this isn't going to be a maximally compact representation of an IP\naddress. Such a thing isn't really possible on the Erlang VM. However,\nstoring the bulk of it as a single integer still lets us perform efficient\nbitwise operations with less overhead than, say, `:inet`-style tuples of\nmultiple integers.","ref":"BitwiseIp.html#t:t/0"},{"type":"type","title":"BitwiseIp.v4/0","doc":"An IPv4 address.\n\nThe `:addr` is an unsigned integer between 0 and 2^32 - 1. See `t:t/0` for\ndiscussion about the in-memory representation.","ref":"BitwiseIp.html#t:v4/0"},{"type":"type","title":"BitwiseIp.v6/0","doc":"An IPv6 address.\n\nThe `:addr` is an unsigned integer between 0 and 2^128 - 1. See `t:t/0` for\ndiscussion about the in-memory representation.","ref":"BitwiseIp.html#t:v6/0"},{"type":"module","title":"BitwiseIp.Block","doc":"A struct representing a range of bitwise IP addresses.\n\nSince 1993, [classless inter-domain routing\n(CIDR)](https://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing) has\nbeen the basis for allocating blocks of IP addresses and efficiently routing\nbetween them.\n\nIf you think about the standard human-readable notation for IP addresses, a\nCIDR block is essentially a pattern with \"wildcards\" at the end. For example,\n`1.2.3.x` would contain the 256 different IPv4 addresses ranging from\n`1.2.3.0` through `1.2.3.255`. The CIDR representation would use the starting\naddress `1.2.3.0` plus a bitmask where the first three bytes (the\nnon-wildcards) are all ones. In IPv4 notation, the mask would be\n`255.255.255.0`. But rather than use wildcards, CIDR blocks have their own\nnotation consisting of the starting address, a slash (`/`), and a prefix\nlength - the number of leading ones in the mask. So the `1.2.3.x` block would\nactually be written as `1.2.3.0/24`.\n\nAs the basis for modern IP routing, these blocks are commonly used as virtual\ncollections. The CIDR representation allows us to efficiently test an\nincoming IP address for membership in the block by bitwise `AND`-ing the mask\nwith the incoming address and comparing the result to the block's starting\naddress. The size of the block can also be computed in constant time using\nbitwise arithmetic on the mask. For example, from the `/24` IPv4 mask we\ncould infer there are 2^8 = 256 addresses in the range corresponding to the\nremaining 8 least significant bits.\n\nUsing this foundation, `BitwiseIp.Block` is able to implement the\n`Enumerable` protocol with `BitwiseIp` structs as members. This allows you to\nmanipulate blocks as generic collections without actually allocating an\nentire list:\n\n```\niex> :rand.seed(:exs1024, {0, 0, 0})\niex> BitwiseIp.Block.parse!(\"1.2.3.0/24\") |> Enum.random() |> to_string()\n\"1.2.3.115\"\n\niex> BitwiseIp.Block.parse!(\"1.2.3.0/30\") |> Enum.map(&to_string/1)\n[\"1.2.3.0\", \"1.2.3.1\", \"1.2.3.2\", \"1.2.3.3\"]\n```\n\nNote that, while CIDR blocks are efficient on their own, they're locked into\nthis very specific prefix representation. For example, you couldn't represent\nthe range `1.2.3.10` through `1.2.3.20` with a single block, since the binary\nrepresentation isn't amenable to a single prefix. This means you typically\nhave to manipulate multiple blocks at a time. To ensure lists of blocks are\nhandled efficiently, use the `BitwiseIp.Blocks` module.","ref":"BitwiseIp.Block.html"},{"type":"function","title":"BitwiseIp.Block.member?/2","doc":"Efficiently checks if a bitwise IP is within a block.\n\nIn effect, we're testing if the given IP address has the same prefix as the\nblock. This involves a single bitwise `AND` and an integer comparison. We\nextract the prefix from the IP by applying the block's bitmask, then check if\nit's equal to the block's starting address. If the block and the IP have\ndifferent protocols, this function will return `false`.\n\nBecause `BitwiseIp.Block` implements the `Enumerable` protocol, you may also\nuse `in/2` to test for membership.","ref":"BitwiseIp.Block.html#member?/2"},{"type":"function","title":"Examples - BitwiseIp.Block.member?/2","doc":"```\niex> BitwiseIp.Block.parse!(\"192.168.0.0/16\")\n...> |> BitwiseIp.Block.member?(BitwiseIp.parse!(\"192.168.10.1\"))\ntrue\n\niex> BitwiseIp.Block.parse!(\"192.168.0.0/16\")\n...> |> BitwiseIp.Block.member?(BitwiseIp.parse!(\"172.16.0.1\"))\nfalse\n\niex> BitwiseIp.parse!(\"d:e:a:d:b:e:e:f\") in BitwiseIp.Block.parse!(\"d::/16\")\ntrue\n\niex> BitwiseIp.parse!(\"127.0.0.1\") in BitwiseIp.Block.parse!(\"::/0\")\nfalse\n```","ref":"BitwiseIp.Block.html#member?/2-examples"},{"type":"function","title":"BitwiseIp.Block.parse/1","doc":"Parses a bitwise IP block from a string in CIDR notation.\n\nThis function parses strings in [CIDR\nnotation](https://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing#CIDR_notation),\nwhere an IP address is followed by a prefix length composed of a slash (`/`)\nand a decimal number of leading bits in the subnet mask. The prefix length is\noptional. If missing, it defaults to the full width of the IP address: 32\nbits for IPv4, 128 for IPv6.\n\nThe constituent parts are parsed using `BitwiseIp.parse/1` and\n`BitwiseIp.Mask.parse/2`. The address has the mask applied before\nconstructing the `BitwiseIp.Block` struct, thereby discarding any lower bits.\nThis parsing is done in an error-safe way by returning a tagged tuple. To\nraise an error, use `parse!/1` instead.\n\n`BitwiseIp.Block` implements the `String.Chars` protocol, so parsing can be\nundone using `to_string/1`.","ref":"BitwiseIp.Block.html#parse/1"},{"type":"function","title":"Examples - BitwiseIp.Block.parse/1","doc":"```\niex> BitwiseIp.Block.parse(\"192.168.0.0/16\")\n{:ok, %BitwiseIp.Block{proto: :v4, addr: 3232235520, mask: 4294901760}}\n\niex> BitwiseIp.Block.parse(\"fc00::/8\")\n{:ok, %BitwiseIp.Block{proto: :v6, addr: 334965454937798799971759379190646833152, mask: 338953138925153547590470800371487866880}}\n\niex> BitwiseIp.Block.parse(\"256.0.0.0/8\")\n{:error, \"Invalid IP address \\\"256.0.0.0\\\" in CIDR \\\"256.0.0.0/8\\\"\"}\n\niex> BitwiseIp.Block.parse(\"dead::beef/129\")\n{:error, \"Invalid IPv6 mask \\\"129\\\" in CIDR \\\"dead::beef/129\\\"\"}\n\niex> BitwiseIp.Block.parse(\"192.168.0.0/8\") |> elem(1) |> to_string()\n\"192.0.0.0/8\"\n\niex> BitwiseIp.Block.parse(\"::\") |> elem(1) |> to_string()\n\"::/128\"\n```","ref":"BitwiseIp.Block.html#parse/1-examples"},{"type":"function","title":"BitwiseIp.Block.parse!/1","doc":"An error-raising variant of `parse/1`.\n\nThis function parses strings in [CIDR\nnotation](https://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing#CIDR_notation),\nwhere an IP address is followed by a prefix length composed of a slash (`/`)\nand a decimal number of leading bits in the subnet mask. The prefix length is\noptional. If missing, it defaults to the full width of the IP address: 32\nbits for IPv4, 128 for IPv6.\n\nThe constituent parts are parsed using `BitwiseIp.parse/1` and\n`BitwiseIp.Mask.parse/2`. The address has the mask applied before\nconstructing the `BitwiseIp.Block` struct, thereby discarding any lower bits.\nIf the string is invalid, this function raises an `ArgumentError`.\n\n`BitwiseIp.Block` implements the `String.Chars` protocol, so parsing can be\nundone using `to_string/1`.","ref":"BitwiseIp.Block.html#parse!/1"},{"type":"function","title":"Examples - BitwiseIp.Block.parse!/1","doc":"```\niex> BitwiseIp.Block.parse!(\"192.168.0.0/16\")\n%BitwiseIp.Block{proto: :v4, addr: 3232235520, mask: 4294901760}\n\niex> BitwiseIp.Block.parse!(\"fc00::/8\")\n%BitwiseIp.Block{proto: :v6, addr: 334965454937798799971759379190646833152, mask: 338953138925153547590470800371487866880}\n\niex> BitwiseIp.Block.parse!(\"256.0.0.0/8\")\n** (ArgumentError) Invalid IP address \"256.0.0.0\" in CIDR \"256.0.0.0/8\"\n\niex> BitwiseIp.Block.parse!(\"dead::beef/129\")\n** (ArgumentError) Invalid IPv6 mask \"129\" in CIDR \"dead::beef/129\"\n\niex> BitwiseIp.Block.parse!(\"192.168.0.0/8\") |> to_string()\n\"192.0.0.0/8\"\n\niex> BitwiseIp.Block.parse!(\"::\") |> to_string()\n\"::/128\"\n```","ref":"BitwiseIp.Block.html#parse!/1-examples"},{"type":"function","title":"BitwiseIp.Block.size/1","doc":"Computes the number of addresses contained in a block.\n\nThis value is wholly determined by the `:mask` field. Taking the bitwise\ncomplement of the mask gives us an unsigned integer where all the lower bits\nare ones. Since these are the bits that are covered by the block, we can\ninterpret this as the number of possible values, minus one for the zeroth\naddress.\n\nFor example, the IPv4 prefix `/29` leaves 3 bits to represent different\naddresses in the block. So that's 2^3 = 8 possible addresses. To get there\nfrom the mask `0b11111111111111111111111111111000`, we take its complement\nand get `0b00000000000000000000000000000111`, which represents the integer\n2^3 - 1 = 7. We add 1 and get the 8 possible addresses.\n\nBecause of the limited number of possible masks, we might want to implement\nthis as a static lookup using pattern matched function clauses, thereby\navoiding binary manipulation altogether. However, benchmarks indicate that\npattern matching against structs is much slower than the required bitwise\nmath. So, we negate the mask and add 1 to the resulting integer at run time.","ref":"BitwiseIp.Block.html#size/1"},{"type":"function","title":"Examples - BitwiseIp.Block.size/1","doc":"```\niex> BitwiseIp.Block.parse!(\"1.2.3.4/32\") |> BitwiseIp.Block.size()\n1\niex> BitwiseIp.Block.parse!(\"1.2.3.4/31\") |> BitwiseIp.Block.size()\n2\niex> BitwiseIp.Block.parse!(\"1.2.3.4/30\") |> BitwiseIp.Block.size()\n4\niex> BitwiseIp.Block.parse!(\"1.2.3.4/29\") |> BitwiseIp.Block.size()\n8\n\niex> BitwiseIp.Block.parse!(\"::/124\") |> BitwiseIp.Block.size()\n16\niex> BitwiseIp.Block.parse!(\"::/123\") |> BitwiseIp.Block.size()\n32\niex> BitwiseIp.Block.parse!(\"::/122\") |> BitwiseIp.Block.size()\n64\niex> BitwiseIp.Block.parse!(\"::/121\") |> BitwiseIp.Block.size()\n128\n```","ref":"BitwiseIp.Block.html#size/1-examples"},{"type":"function","title":"BitwiseIp.Block.subnet?/2","doc":"Efficiently checks if `block2` is a subset of `block1`.\n\nThanks to `BitwiseIp.Mask`, we encode masks as integers. So if mask A is less\nthan mask B, that means A had fewer leading bits, meaning the block will\ncontain *more* addresses than the block for B. Therefore, as a prerequisite,\nwe first check that `block1`'s mask is `<=` `block2`'s mask. If not, then\nthere's no chance that `block2` could be wholly contained in `block1`.\n\nThen, if `block1`'s range is wide enough, we can test an arbitrary IP from\n`block2` for membership in `block1`. Its inclusion would imply that\neverything else in `block2` is also included, since `block1` is wider. We\nhave a suitable address to test in the form of the `:addr` field from\n`block2`. The membership check involves the same bitwise `AND` + integer\ncomparison as `member?/2`.\n\nIf the blocks don't have matching protocols, this function returns `false`.","ref":"BitwiseIp.Block.html#subnet?/2"},{"type":"function","title":"Examples - BitwiseIp.Block.subnet?/2","doc":"```\niex> BitwiseIp.Block.parse!(\"1.0.0.0/8\")\n...> |> BitwiseIp.Block.subnet?(BitwiseIp.Block.parse!(\"1.2.0.0/16\"))\ntrue\n\niex> BitwiseIp.Block.parse!(\"1.2.0.0/16\")\n...> |> BitwiseIp.Block.subnet?(BitwiseIp.Block.parse!(\"1.0.0.0/8\"))\nfalse\n\niex> BitwiseIp.Block.parse!(\"1.2.0.0/16\")\n...> |> BitwiseIp.Block.subnet?(BitwiseIp.Block.parse!(\"1.2.0.0/16\"))\ntrue\n\niex> BitwiseIp.Block.parse!(\"1.2.0.0/16\")\n...> |> BitwiseIp.Block.subnet?(BitwiseIp.Block.parse!(\"2.3.0.0/16\"))\nfalse\n```","ref":"BitwiseIp.Block.html#subnet?/2-examples"},{"type":"type","title":"BitwiseIp.Block.t/0","doc":"A bitwise IP address block.\n\nThe block consists of all IP addresses that share the same prefix. To\nrepresent this, we use a struct with the following fields:\n\n* `:proto` - the protocol, either `:v4` or `:v6`\n* `:addr` - the integer encoding of the network prefix\n* `:mask` - the integer encoding of the subnet mask\n\nLogically, this type is a combination of `t:BitwiseIp.t/0` and an integer\nencoded by `BitwiseIp.Mask.encode/2`. However, rather than hold onto a\nliteral `BitwiseIp` struct, the `:proto` and `:addr` fields are inlined. This\nproves to be more efficient for pattern matching than using a nested struct.\n\nThe network prefix's least significant bits are all assumed to be zero,\neffectively making it the starting address of the block. That way, we can\navoid performing repetitive bitwise `AND` operations between the prefix &\nmask in functions such as `member?/2`.","ref":"BitwiseIp.Block.html#t:t/0"},{"type":"type","title":"BitwiseIp.Block.v4/0","doc":"An IPv4 block.\n\nThe `:proto` and `:addr` are the same as in `t:BitwiseIp.v4/0`. The mask is a\n32-bit unsigned integer where some number of leading bits are one and the\nrest are zero. See `t:t/0` for more details.","ref":"BitwiseIp.Block.html#t:v4/0"},{"type":"type","title":"BitwiseIp.Block.v6/0","doc":"An IPv6 block.\n\nThe `:proto` and `:addr` are the same as in `t:BitwiseIp.v6/0`. The mask is a\n128-bit unsigned integer where some number of leading bits are one and the\nrest are zero. See `t:t/0` for more details.","ref":"BitwiseIp.Block.html#t:v6/0"},{"type":"module","title":"BitwiseIp.Blocks","doc":"Functions for handling lists of bitwise IP blocks.\n\nBecause the `BitwiseIp.Block` representation relies on a binary prefix, it's\nnot possible to express certain ranges with a single block. For instance, the\nrange of addresses between `192.168.12.0` and `192.168.16.255` might make\nintuitive sense, but the binary representation of the third byte presents a\nchallenge:\n\n* `12` = `0b00001100`\n* `13` = `0b00001101`\n* `14` = `0b00001110`\n* `15` = `0b00001111`\n* `16` = `0b00010000`\n\nNotice that `12`-`15` share the prefix `0b000011xx`, so those addresses could\nbe covered by the CIDR block `192.168.12.0/22`. (The prefix length is 22 for\nthe 16 bits of `192.168.` plus the 6 most significant bits of the third\nbyte.) But that would *not* cover the `192.168.16.x` addresses:\n\n```\niex> BitwiseIp.Block.parse!(\"192.168.12.0/22\")\n...> |> Enum.take_every(256)\n...> |> Enum.map(&to_string/1)\n[\"192.168.12.0\", \"192.168.13.0\", \"192.168.14.0\", \"192.168.15.0\"]\n```\n\nAll this is to say that there are general limitations to the expressiveness\nof a single CIDR range, so it's natural that most applications will deal with\na collection of blocks at a time - conceptually, a list of lists of IP\naddresses.\n\nWhereas bitwise IP blocks have a straightforward binary representation, a\nlist of blocks is somewhat more unwieldy. This module provides utility\nfunctions that make handling these lists more ergonomic. In particular, the\n`member?/2` function helps you avoid a common performance pitfall.","ref":"BitwiseIp.Blocks.html"},{"type":"function","title":"BitwiseIp.Blocks.member?/2","doc":"Efficiently checks if an IP address is a member of any of the blocks.\n\nLibraries will generally handle IP addresses encoded as `:inet`-style tuples\nof integers. Therefore, in order to use `BitwiseIp.Block.member?/2`, you'll\nfirst need to use `BitwiseIp.encode/1` to convert the tuple into an\ninteger-encoded struct.\n\nA common mistake when handling a list of blocks is to do the bitwise IP\nencoding repeatedly within a loop:\n\n```\n# This is a mistake!\nip = {127, 0, 0, 1}\nEnum.any?(blocks, &BitwiseIp.Block.member?(&1, BitwiseIp.encode(ip)))\n```\n\nThe problem with the above is that the return value of `BitwiseIp.encode(ip)`\ndoesn't change as we iterate through the list. The cost of redundantly\nencoding the same IP address over & over is often enough to outweigh any\nperformance gains from using the bitwise membership checks.\n\nThis function helps enforce a pattern where the encoding is only done once\n(essentially performing [loop-invariant code\nmotion](https://en.wikipedia.org/wiki/Loop-invariant_code_motion)). That is,\nit's akin to saying:\n\n```\nip = {127, 0, 0, 1}\nencoded = BitwiseIp.encode(ip) # this is only done once\nEnum.any?(blocks, &BitwiseIp.Block.member?(&1, encoded))\n```\n\nThis function also accepts an already-encoded `BitwiseIp` struct as an\nargument, in which case no extra encoding needs to be performed. This is\nuseful for cases where you need to perform even more loop-invariant code\nmotion, such as when you're handling two separate lists. In such a case, you\nshould use a pattern like:\n\n```\n# make sure to only encode the IP once\nip = {127, 0, 0, 1}\nencoded = BitwiseIp.encode(ip)\n\nBitwiseIp.Blocks.member?(blocks1, encoded) # check the first list\nBitwiseIp.Blocks.member?(blocks2, encoded) # check the second list\n```","ref":"BitwiseIp.Blocks.html#member?/2"},{"type":"function","title":"Examples - BitwiseIp.Blocks.member?/2","doc":"```\niex> [\"1.2.0.0/16\", \"3.4.0.0/16\", \"5.6.0.0/16\"]\n...> |> Enum.map(&BitwiseIp.Block.parse!/1)\n...> |> BitwiseIp.Blocks.member?({1, 2, 3, 4})\ntrue\n\niex> [\"1.2.0.0/16\", \"3.4.0.0/16\", \"5.6.0.0/16\"]\n...> |> Enum.map(&BitwiseIp.Block.parse!/1)\n...> |> BitwiseIp.Blocks.member?({7, 8, 9, 10})\nfalse\n\niex> [\"1.2.0.0/16\", \"3.4.0.0/16\", \"5.6.0.0/16\"]\n...> |> Enum.map(&BitwiseIp.Block.parse!/1)\n...> |> BitwiseIp.Blocks.member?(BitwiseIp.encode({1, 2, 3, 4}))\ntrue\n\niex> [\"1.2.0.0/16\", \"3.4.0.0/16\", \"5.6.0.0/16\"]\n...> |> Enum.map(&BitwiseIp.Block.parse!/1)\n...> |> BitwiseIp.Blocks.member?(BitwiseIp.encode({7, 8, 9, 10}))\nfalse\n```","ref":"BitwiseIp.Blocks.html#member?/2-examples"},{"type":"function","title":"BitwiseIp.Blocks.optimize/1","doc":"Computes an equivalent list of blocks optimal for `member?/2`.\n\nWhile an individual `BitwiseIp.Block.member?/2` call is already efficient,\nthe performance of `member?/2` is sensitive to a couple of factors:\n\n1. The size of the list matters, since a smaller list requires fewer\n   individual checks.\n\n2. The order of the elements in the list matters, since `member?/2` will exit\n   early as soon as any individual check returns true.\n\nTo optimize for the size of the list, this function recursively merges any\ntwo blocks where one is a subset of the other. This is tested using\n`BitwiseIp.Block.subnet?/2`. For example, `1.2.0.0/16` is a subset of\n`1.0.0.0/8`, so instead of calling `BitwiseIp.Block.member?/2` on both of\nthem, we can simply check the larger range of the two - in this case,\n`1.0.0.0/8`.\n\nThe order can be optimized by placing larger blocks earlier in the list.\nAssuming an even distribution of IP addresses, it's more likely for an\naddress to fall inside of a block that covers a wider range. Thus, we can\nsort by the integer-encoded mask: a smaller mask means a shorter network\nprefix, which means there are more addresses possible (see\n`BitwiseIp.Block.size/1` for more on computing the size of a block from its\nmask).\n\nThis optimization is kind of a parlor trick cribbed from the\n[cider](https://hex.pm/packages/cider) library. Except in pathological cases,\nthe run time cost of performing the optimization is likely larger than any\nperformance gained by using the new list. As such, if you're going to use\nthis function at all, it's only really appropriate to call at compile time,\nwhich means your original list of blocks has to be available statically.","ref":"BitwiseIp.Blocks.html#optimize/1"},{"type":"function","title":"Examples - BitwiseIp.Blocks.optimize/1","doc":"```\niex> [\"1.2.3.4\", \"1.2.3.0/24\", \"1.2.0.0/16\", \"1.0.0.0/8\"]\n...> |> BitwiseIp.Blocks.parse!()\n...> |> BitwiseIp.Blocks.optimize()\n...> |> Enum.map(&to_string/1)\n[\"1.0.0.0/8\"]\n\niex> [\"1.2.0.0/16\", \"3.0.0.0/8\"]\n...> |> BitwiseIp.Blocks.parse!()\n...> |> BitwiseIp.Blocks.optimize()\n...> |> Enum.map(&to_string/1)\n[\"3.0.0.0/8\", \"1.2.0.0/16\"]\n\niex> [\"1.2.0.0/16\", \"3.4.5.0/24\", \"1.0.0.0/8\", \"3.4.0.0/16\"]\n...> |> BitwiseIp.Blocks.parse!()\n...> |> BitwiseIp.Blocks.optimize()\n...> |> Enum.map(&to_string/1)\n[\"1.0.0.0/8\", \"3.4.0.0/16\"]\n```","ref":"BitwiseIp.Blocks.html#optimize/1-examples"},{"type":"function","title":"BitwiseIp.Blocks.parse/1","doc":"Parses a list of strings into bitwise IP blocks.\n\nThis function takes a list of strings in CIDR notation and parses them into\nbitwise IP blocks using `BitwiseIp.Block.parse/1`. If a string is invalid,\nits value is discarded from the resulting list. If you want to raise an error\ninstead, use `parse!/1`.","ref":"BitwiseIp.Blocks.html#parse/1"},{"type":"function","title":"Examples - BitwiseIp.Blocks.parse/1","doc":"```\niex> BitwiseIp.Blocks.parse([\"3.14.0.0/16\", \"dead::beef\"])\n...> |> Enum.map(&to_string/1)\n[\"3.14.0.0/16\", \"dead::beef/128\"]\n\niex> BitwiseIp.Blocks.parse([\"3.14/16\", \"invalid\", \"dead::cow\"])\n[]\n\niex> BitwiseIp.Blocks.parse([\"3.14.0.0/16\", \"invalid\", \"dead::beef\"])\n...> |> Enum.map(&to_string/1)\n[\"3.14.0.0/16\", \"dead::beef/128\"]\n```","ref":"BitwiseIp.Blocks.html#parse/1-examples"},{"type":"function","title":"BitwiseIp.Blocks.parse!/1","doc":"An error-raising variant of `parse/1`.\n\nThis function takes a list of strings in CIDR notation and parses them into\nbitwise IP blocks using `BitwiseIp.Block.parse!/1`. If any of the strings are\ninvalid, the whole list fails to parse and the error is propagated. If you\nwant to discard invalid elements instead, use `parse/1`.","ref":"BitwiseIp.Blocks.html#parse!/1"},{"type":"function","title":"Examples - BitwiseIp.Blocks.parse!/1","doc":"```\niex> BitwiseIp.Blocks.parse!([\"3.14.0.0/16\", \"dead::beef\"])\n...> |> Enum.map(&to_string/1)\n[\"3.14.0.0/16\", \"dead::beef/128\"]\n\niex> BitwiseIp.Blocks.parse!([\"3.14/16\", \"invalid\", \"dead::cow\"])\n** (ArgumentError) Invalid IP address \"3.14\" in CIDR \"3.14/16\"\n\niex> BitwiseIp.Blocks.parse!([\"3.14.0.0/16\", \"invalid\", \"dead::beef\"])\n** (ArgumentError) Invalid IP address \"invalid\" in CIDR \"invalid\"\n```","ref":"BitwiseIp.Blocks.html#parse!/1-examples"},{"type":"type","title":"BitwiseIp.Blocks.t/0","doc":"A list of bitwise IP blocks.\n\nThe `BitwiseIp.Blocks` module operates over lists of `BitwiseIp.Block`\nstructs. This itself does not warrant a separate struct with any extra\nindirection, so we just use lists directly.","ref":"BitwiseIp.Blocks.html#t:t/0"},{"type":"module","title":"BitwiseIp.Mask","doc":"Functions for handling CIDR prefix lengths as bitmasks.\n\nThese functions are used internally by `BitwiseIp.Block` to parse CIDR\nnotation. For example, the IPv4 CIDR prefix length `/12` corresponds to an\nunsigned 32-bit integer of 12 ones followed by 20 zeroes:\n`0b11111111111100000000000000000000`. This mask is used in a bitwise `AND`\nwith an integer-encoded IPv4 address to extract the first 12 bits.\n\nIn IPv6, the same prefix is an unsigned 128-bit integer of 12 ones followed\nby 116 zeroes. Because Elixir's integers don't have a fixed width, we must\ndistinguish between IPv4 and IPv6 in the function signatures, similar to the\n`:proto` tag in the `t:BitwiseIp.t/0` struct.\n\nSince there's a limited domain & range for these functions, they're all\ncompiled directly into function clauses to perform static lookups. There is\nno work done at run time to convert strings, perform bitwise math, or\nanything other than the tacit function dispatch.","ref":"BitwiseIp.Mask.html"},{"type":"function","title":"BitwiseIp.Mask.decode/2","doc":"Decodes a bitmask into an integer prefix length.\n\nGiven the protocol (either `:v4` or `:v6`) and a valid bitmask for that\nprotocol, this function looks up the number of leading ones used by the\nbitmask. The function is only defined on valid IPv4 and IPv6 bitmasks. To\nundo this conversion, use `encode/2`.","ref":"BitwiseIp.Mask.html#decode/2"},{"type":"function","title":"Examples - BitwiseIp.Mask.decode/2","doc":"```\niex> BitwiseIp.Mask.decode(:v4, 0b11111111111100000000000000000000)\n12\n\niex> BitwiseIp.Mask.decode(:v6, 0b11111111111100000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000)\n12\n\niex> BitwiseIp.Mask.decode(:v4, 0b11111111111100000000000000000001)\n** (FunctionClauseError) no function clause matching in BitwiseIp.Mask.decode/2\n\niex> BitwiseIp.Mask.decode(:v6, 0b0101)\n** (FunctionClauseError) no function clause matching in BitwiseIp.Mask.decode/2\n```","ref":"BitwiseIp.Mask.html#decode/2-examples"},{"type":"function","title":"BitwiseIp.Mask.encode/2","doc":"Encodes an integer prefix length as a bitmask.\n\nGiven the protocol (either `:v4` or `:v6`) and the number of leading ones in\nthe prefix, this function looks up the corresponding bitmask. The function is\nonly defined on valid prefix lengths: between 0 and 32 for IPv4 and between 0\nand 128 for IPv6. To undo this conversion, use `decode/2`.","ref":"BitwiseIp.Mask.html#encode/2"},{"type":"function","title":"Examples - BitwiseIp.Mask.encode/2","doc":"```\niex> BitwiseIp.Mask.encode(:v4, 12)\n4293918720\n\niex> BitwiseIp.Mask.encode(:v6, 12)\n340199290171201906221318119490500689920\n\niex> BitwiseIp.Mask.encode(:v4, 128)\n** (FunctionClauseError) no function clause matching in BitwiseIp.Mask.encode/2\n\niex> BitwiseIp.Mask.encode(:v6, -12)\n** (FunctionClauseError) no function clause matching in BitwiseIp.Mask.encode/2\n```","ref":"BitwiseIp.Mask.html#encode/2-examples"},{"type":"function","title":"BitwiseIp.Mask.parse/2","doc":"Parses a string prefix length into a bitmask.\n\nGiven the protocol (either `:v4` or `:v6`) and the string representation of a\nprefix length (without the leading slash), this function looks up the\ncorresponding bitmask. This is done in an error-safe way by returning a\ntagged tuple. To raise an error, use `parse!/2` instead.","ref":"BitwiseIp.Mask.html#parse/2"},{"type":"function","title":"Examples - BitwiseIp.Mask.parse/2","doc":"```\niex> BitwiseIp.Mask.parse(:v4, \"12\")\n{:ok, 4293918720}\n\niex> BitwiseIp.Mask.parse(:v6, \"12\")\n{:ok, 340199290171201906221318119490500689920}\n\niex> BitwiseIp.Mask.parse(:v4, \"128\")\n{:error, \"Invalid IPv4 mask \\\"128\\\"\"}\n\niex> BitwiseIp.Mask.parse(:v6, \"not a mask\")\n{:error, \"Invalid IPv6 mask \\\"not a mask\\\"\"}\n```","ref":"BitwiseIp.Mask.html#parse/2-examples"},{"type":"function","title":"BitwiseIp.Mask.parse!/2","doc":"An error-raising variant of `parse/2`.\n\nGiven the protocol (either `:v4` or `:v6`) and the string representation of a\nprefix length (without the leading slash), this function looks up the\ncorresponding bitmask. If the string cannot be parsed, it raises an\n`ArgumentError`.","ref":"BitwiseIp.Mask.html#parse!/2"},{"type":"function","title":"Examples - BitwiseIp.Mask.parse!/2","doc":"```\niex> BitwiseIp.Mask.parse!(:v4, \"12\")\n4293918720\n\niex> BitwiseIp.Mask.parse!(:v6, \"12\")\n340199290171201906221318119490500689920\n\niex> BitwiseIp.Mask.parse!(:v4, \"128\")\n** (ArgumentError) Invalid IPv4 mask \"128\"\n\niex> BitwiseIp.Mask.parse!(:v6, \"not a mask\")\n** (ArgumentError) Invalid IPv6 mask \"not a mask\"\n```","ref":"BitwiseIp.Mask.html#parse!/2-examples"}],"content_type":"text/markdown"}